@@ -48,8 +48,8 @@ C<PPI::Document> implements the necessary C<STORABLE_freeze> and
 C<STORABLE_thaw> hooks to provide native support for L<Storable>,
 if you have it installed.
 
-However if you want to clone clone a Document, you are highly recommended
-to use the internal C<$Document-E<gt>clone> method rather than Storable's
+However if you want to clone a Document, you are highly recommended
+to use the C<$Document-E<gt>clone> method rather than Storable's
 C<dclone> function (although C<dclone> should still work).
 
 =head1 METHODS
@@ -65,23 +65,20 @@ Document-specific.
 
 use strict;
 use Carp                          ();
-use List::MoreUtils               ();
-use Params::Util                  qw{_SCALAR0 _ARRAY0 _INSTANCE};
+use List::Util 1.33               ();
+use Params::Util 1.00             qw{_SCALAR0 _ARRAY0 _INSTANCE};
 use Digest::MD5                   ();
 use PPI::Util                     ();
 use PPI                           ();
 use PPI::Node                     ();
-use PPI::Exception::ParserTimeout ();
+use YAML::PP                      ();
 
 use overload 'bool' => \&PPI::Util::TRUE;
 use overload '""'   => 'content';
 
-use vars qw{$VERSION @ISA $errstr};
-BEGIN {
-	$VERSION = '1.220';
-	@ISA     = 'PPI::Node';
-	$errstr  = '';
-}
+our $VERSION = '1.281';
+
+our ( $errstr, @ISA ) = ( "", "PPI::Node" );
 
 use PPI::Document::Fragment ();
 
@@ -132,16 +129,57 @@ In all cases, the document is considered to be "anonymous" and not tied back
 to where it was created from. Specifically, if you create a PPI::Document from
 a filename, the document will B<not> remember where it was created from.
 
+Returns a C<PPI::Document> object, or C<undef> if parsing fails.
+L<PPI::Exception> objects can also be thrown if there are parsing problems.
+
 The constructor also takes attribute flags.
 
-At this time, the only available attribute is the C<readonly> flag.
+=head3 readonly
 
-Setting C<readonly> to true will allow various systems to provide
-additional optimisations and caching. Note that because C<readonly> is an
-optimisation flag, it is off by default and you will need to explicitly
-enable it.
+Setting C<readonly> to true will allow various systems to provide additional
+optimisations and caching. Note that because C<readonly> is an optimisation
+flag, it is off by default and you will need to explicitly enable it.
 
-Returns a C<PPI::Document> object, or C<undef> if parsing fails.
+=head3 feature_mods
+
+Setting feature_mods with a hashref allows defining perl parsing features to be
+enabled for the whole document. (e.g. when the code is assumed to be run as a
+oneliner)
+
+=head3 custom_feature_includes
+
+  custom_feature_includes =>
+    { strEct => { signatures => "Syntax::Keyword::Try" } }
+
+Setting custom_feature_includes with a hashref allows defining include names
+which act like pragmas that enable parsing features within their scope.
+
+This is mostly useful when your work project has its own boilerplate module.
+
+It can also be provided as JSON or YAML in the environment variable
+PPI_CUSTOM_FEATURE_INCLUDES, like so:
+
+  PPI_CUSTOM_FEATURE_INCLUDES='strEct: {signatures: perl}' \
+    perlcritic lib/OurModule.pm
+
+  PPI_CUSTOM_FEATURE_INCLUDES='{"strEct":{"signatures":"perl"}}' \
+    perlcritic lib/OurModule.pm
+
+=head3 custom_feature_include_cb
+
+  custom_feature_include_cb => sub {
+    my ($statement) = @_;
+    return $statement->module eq "strEct" ? { signatures => "perl" } : ();
+  },
+
+Setting custom_feature_include_cb with a code reference causes all inspections
+on includes to call that sub before doing any other inspections. The sub can
+decide to either return a hashref of features to be enabled or disabled, which
+will be used for the scope the include was called in, or undef to continue with
+the default inspections. The argument to the sub will be the
+L<PPI::Statement::Include> object.
+
+This can be useful when your work project has a complex boilerplate module.
 
 =cut
 
@@ -159,10 +197,6 @@ sub new {
 	# Check constructor attributes
 	my $source  = shift;
 	my %attr    = @_;
-	my $timeout = delete $attr{timeout};
-	if ( $timeout and ! PPI::Util::HAVE_ALARM() ) {
-		Carp::croak("This platform does not support PPI parser timeouts");
-	}
 
 	# Check the data source
 	if ( ! defined $source ) {
@@ -174,6 +208,9 @@ sub new {
 			Carp::croak("API CHANGE: Source code should only be passed to PPI::Document->new as a SCALAR reference");
 		}
 
+		# Save the filename
+		$attr{filename} ||= $source;
+
 		# When loading from a filename, use the caching layer if it exists.
 		if ( $CACHE ) {
 			my $file_contents = PPI::Util::_slurp( $source );
@@ -185,64 +222,25 @@ sub new {
 			my $document = $CACHE->get_document($file_contents);
 			return $class->_setattr( $document, %attr ) if $document;
 
-			if ( $timeout ) {
-				eval {
-					local $SIG{ALRM} = sub { die "alarm\n" };
-					alarm( $timeout );
-					$document = PPI::Lexer->lex_source( $$file_contents );
-					alarm( 0 );
-				};
-			} else {
-				$document = PPI::Lexer->lex_source( $$file_contents );
-			}
+			$document = PPI::Lexer->lex_source( $$file_contents, %attr );
 			if ( $document ) {
 				# Save in the cache
 				$CACHE->store_document( $document );
-				return $class->_setattr( $document, %attr );
+				return $document;
 			}
 		} else {
-			if ( $timeout ) {
-				eval {
-					local $SIG{ALRM} = sub { die "alarm\n" };
-					alarm( $timeout );
-					my $document = PPI::Lexer->lex_file( $source );
-					return $class->_setattr( $document, %attr ) if $document;
-					alarm( 0 );
-				};
-			} else {
-				my $document = PPI::Lexer->lex_file( $source );
-				return $class->_setattr( $document, %attr ) if $document;
-			}
+			my $document = PPI::Lexer->lex_file( $source, %attr );
+			return $document if $document;
 		}
 
 	} elsif ( _SCALAR0($source) ) {
-		if ( $timeout ) {
-			eval {
-				local $SIG{ALRM} = sub { die "alarm\n" };
-				alarm( $timeout );
-				my $document = PPI::Lexer->lex_source( $$source );
-				return $class->_setattr( $document, %attr ) if $document;
-				alarm( 0 );
-			};
-		} else {
-			my $document = PPI::Lexer->lex_source( $$source );
-			return $class->_setattr( $document, %attr ) if $document;
-		}
+		my $document = PPI::Lexer->lex_source( $$source, %attr );
+		return $document if $document;
 
 	} elsif ( _ARRAY0($source) ) {
 		$source = join '', map { "$_\n" } @$source;
-		if ( $timeout ) {
-			eval {
-				local $SIG{ALRM} = sub { die "alarm\n" };
-				alarm( $timeout );
-				my $document = PPI::Lexer->lex_source( $source );
-				return $class->_setattr( $document, %attr ) if $document;
-				alarm( 0 );
-			};
-		} else {
-			my $document = PPI::Lexer->lex_source( $source );
-			return $class->_setattr( $document, %attr ) if $document;
-		}
+		my $document = PPI::Lexer->lex_source( $source, %attr );
+		return $document if $document;
 
 	} else {
 		$class->_error("Unknown object or reference was passed to PPI::Document::new");
@@ -250,9 +248,7 @@ sub new {
 
 	# Pull and store the error from the lexer
 	my $errstr;
-	if ( _INSTANCE($@, 'PPI::Exception::Timeout') ) {
-		$errstr = 'Timed out while parsing document';
-	} elsif ( _INSTANCE($@, 'PPI::Exception') ) {
+	if ( _INSTANCE($@, 'PPI::Exception') ) {
 		$errstr = $@->message;
 	} elsif ( $@ ) {
 		$errstr = $@;
@@ -271,8 +267,21 @@ sub load {
 }
 
 sub _setattr {
-	my ($class, $document, %attr) = @_;
-	$document->{readonly} = !! $attr{readonly};
+	my ( $class, $document, %attr ) = @_;
+	$document->{readonly}                  = !!$attr{readonly};
+	$document->{filename}                  = $attr{filename};
+	$document->{feature_mods}              = $attr{feature_mods};
+	$document->{custom_feature_includes}   = $attr{custom_feature_includes};
+	$document->{custom_feature_include_cb} = $attr{custom_feature_include_cb};
+	if ( $ENV{PPI_CUSTOM_FEATURE_INCLUDES} ) {
+		my $includes = YAML::PP::Load $ENV{PPI_CUSTOM_FEATURE_INCLUDES};
+		die "\$ENV{PPI_CUSTOM_FEATURE_INCLUDES} "
+		  . "does not contain valid perl:\n"
+		  . "val: '$ENV{PPI_CUSTOM_FEATURE_INCLUDES}'\nerr: $@"
+		  if $@;
+		$document->{custom_feature_includes} =
+		  { %{ $document->{custom_feature_includes} || {} }, %{$includes} };
+	}
 	return $document;
 }
 
@@ -338,6 +347,19 @@ sub get_cache {
 
 =pod
 
+=head2 filename
+
+The C<filename> accessor returns the name of the file in which the document
+is stored.
+
+=cut
+
+sub filename {
+	$_[0]->{filename};
+}
+
+=pod
+
 =head2 readonly
 
 The C<readonly> attribute indicates if the document is intended to be
@@ -375,6 +397,36 @@ sub tab_width {
 	$self->{tab_width} = shift;
 }
 
+=head2 feature_mods { feature_name => $provider }
+
+=cut
+
+sub feature_mods {
+	my $self = shift;
+	return $self->{feature_mods} unless @_;
+	$self->{feature_mods} = shift;
+}
+
+=head2 custom_feature_includes { module_name => { feature_name => $provider } }
+
+=cut
+
+sub custom_feature_includes {
+	my $self = shift;
+	return $self->{custom_feature_includes} unless @_;
+	$self->{custom_feature_includes} = shift;
+}
+
+=head2 custom_feature_include_cb sub { ... }
+
+=cut
+
+sub custom_feature_include_cb {
+	my $self = shift;
+	return $self->{custom_feature_include_cb} unless @_;
+	$self->{custom_feature_include_cb} = shift;
+}
+
 =pod
 
 =head2 save
@@ -391,6 +443,7 @@ sub save {
 	my $self = shift;
 	local *FILE;
 	open( FILE, '>', $_[0] )    or return undef;
+	binmode FILE;
 	print FILE $self->serialize or return undef;
 	close FILE                  or return undef;
 	return 1;
@@ -458,9 +511,12 @@ sub serialize {
 		# case will definitely not contain a newline.
 		$output .= $Token->content;
 
+		# Pick up the indentation, which may be undef.
+		my $indentation = $Token->indentation || '';
+
 		# Now add all of the here-doc content to the heredoc buffer.
 		foreach my $line ( $Token->heredoc ) {
-			$heredoc .= $line;
+			$heredoc .= "\n" eq $line ? $line : $indentation . $line;
 		}
 
 		if ( $Token->{_damaged} ) {
@@ -483,7 +539,7 @@ sub serialize {
 			# This is a two part test.
 			# First, are we on the last line of the
 			# content part of the file
-			my $last_line = List::MoreUtils::none {
+			my $last_line = List::Util::none {
 				$tokens[$_] and $tokens[$_]->{content} =~ /\n/
 				} (($i + 1) .. $last_index);
 			if ( ! defined $last_line ) {
@@ -493,7 +549,7 @@ sub serialize {
 
 			# Secondly, are their any more here-docs after us,
 			# (with content or a terminator)
-			my $any_after = List::MoreUtils::any {
+			my $any_after = List::Util::any {
 				$tokens[$_]->isa('PPI::Token::HereDoc')
 				and (
 					scalar(@{$tokens[$_]->{_heredoc}})
@@ -524,7 +580,7 @@ sub serialize {
 
 		# Now add the termination line to the heredoc buffer
 		if ( defined $Token->{_terminator_line} ) {
-			$heredoc .= $Token->{_terminator_line};
+			$heredoc .= $indentation . $Token->{_terminator_line};
 		}
 	}
 
@@ -590,7 +646,7 @@ sub index_locations {
 	my @tokens = $self->tokens;
 
 	# Whenever we hit a heredoc we will need to increment by
-	# the number of lines in it's content section when we
+	# the number of lines in its content section when we
 	# encounter the next token with a newline in it.
 	my $heredoc = 0;
 
@@ -791,7 +847,7 @@ Returns a L<PPI::Document::Normalized> object, or C<undef> on error.
 sub normalized {
 	# The normalization process will utterly destroy and mangle
 	# anything passed to it, so we are going to only give it a
-	# clone of ourself.
+	# clone of ourselves.
 	PPI::Normal->process( $_[0]->clone );
 }
 
@@ -941,7 +997,7 @@ Adam Kennedy E<lt>adamk@cpan.orgE<gt>
 
 =head1 SEE ALSO
 
-L<PPI>, L<http://ali.as/>
+L<PPI>, L<https://web.archive.org/web/20230911221703/http://ali.as/>
 
 =head1 COPYRIGHT
 
